/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * MXcleaner.h (MXcleaner)
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* MXcleaner.h */
#ifndef	_MXCLEANER_H
#define	_MXCLEANER_H

#include "os.h"
#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS 64
/* #define __MSVCRT_VERSION__  0x0801 */

#ifdef OS_LINUX
#define STR_HOME "HOME"
#define STR_DRIVE ""
#define SLASH '/'
#define _MAX_PATH 4096
#define _MAX_FNAME 256
#define PROGRAM_NAME "MXcleaner"
#endif

#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <limits.h>

#ifndef MX
#define MX
#endif

#ifndef VERSION
#define VERSION "2.0.0"
#endif

#ifndef AUTHORS
#define AUTHORS "Elmar Haag"
#endif

#ifndef FALSE
#      define FALSE (0)
#endif

#ifndef TRUE
#      define TRUE (1)
#endif

#ifndef START
#   define START 0
#endif

#ifndef STOP
#   define STOP 1
#endif

#ifdef MX
#   define AUDIO      0
#   define VIDEO      1
#   define EXT_LEN    4
#   define PLAY_TIME  4
#   define FREQ       4
#   define BITRATE    2
#   define FIL_LEN    4
#   define HASH_LEN  32
#endif

#ifndef _FILE_STATUS
#define _FILE_STATUS
#   define NON 0 /* No status */
#   define INC 1 /* __INCOMPLETE___ file */
#   define MXD 2 /* Double MX file */
#   define DBL 3 /* Double complete file */
#   define DIF 4 /* Complete file with Differences */
#   define NDB 5 /* Single complete file */
#   define NIL 6 /* Zero lenght file */
#   define CRC 7 /* CRC with or to be made */
#   define ERR 8 /* Error */
#endif

#ifndef _FLAG_STATUS
#define _FLAG_STATUS
#   define QUIET   0
#   define CLEAN   1
#   define DEL     2
#   define PCT     3
#   define REMOVE  4
#   define TEST    5
#   define VERB    6
#   define HELP    7
#   define IGNORE  8
#   define KILL    9
#   define UPDATE  10
#   define SAVE    11
#   define FLAGS   6
#   define CFLAGS  13
#endif

#ifndef BUF_SIZE
#   define BLK_SIZE 0x2000
#   define ULLI_SIZE sizeof(unsigned long long int)
#   define BUF_SIZE (BLK_SIZE * ULLI_SIZE)
#endif
/*
#ifndef int64
#    define int64 __int64
#    define uint64 unsigned __int64
#endif
*/
#define FACTOR 10

#ifndef errno_t
#define errno_t int
#endif

typedef struct
{
    unsigned long long int quot;
    unsigned long long int rem;
} ull_div_t;

typedef struct
{
    int path_no;
    int plen;
    char path[_MAX_PATH];
} pathstruct;

typedef struct
{
    long long int flen;
    int path_no;
    int rec_no;
    int mx_rec_no;
    int flag;
    unsigned long long int hash;
    unsigned int mx_hash[4];
    char fname[_MAX_FNAME];
} dirstruct;

#ifndef _FULL_PATH
#define _FULL_PATH (sizeof(pathstruct) + sizeof(dirstruct))
#endif

extern char fl[9][4];
extern char *pt;
extern char *tmp;

int print (const char *format, ...);
void print_marked_file();
void print_debug(char *, int, int );
char *strcpy_s(char *, unsigned int, char *);
char *strcat_s(char *, unsigned int, char *);

void *xcalloc(size_t);
void *xmalloc (size_t);
void *xrealloc (void *, size_t);
ull_div_t ulldiv (unsigned long long int, unsigned long long int);


#endif /* _MXCLEANER_H */

