/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * cmp_files.c
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MXcleaner.h"
#include "disp.h"
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>

// int lstat64 (const char *, struct stat64 *);

extern char *path;
extern dirstruct *a;
extern pathstruct *p;
extern char *program_name;

int cmp_file(int, int, int);
int read_file(int, void *, int);
void close_file(int);


int cmp_file(int x, int y, int pct)
{
    int f1 = 0;
    int f2 = 0;
    int ret = 0;

    int buf_size = BUF_SIZE;
    unsigned long long int ctn = 0;
    unsigned long long int flen = 0;
    unsigned int *intbuf1 = NULL;
    unsigned int *intbuf2 = NULL;
    unsigned char *charbuf1 = NULL;
    unsigned char *charbuf2 = NULL;
    long long int aa, m, bm, bs = 0;
    long long int br;
    ldiv_t xb;
    ull_div_t z;

    z = ulldiv(flen, BUF_SIZE);
    br = z.quot;
    bs = BUF_SIZE;
    xb = ldiv(BUF_SIZE, ULLI_SIZE);
    bm = xb.quot;

    intbuf1 = (unsigned int *) xmalloc(buf_size + 8);
    intbuf2 = (unsigned int *) xmalloc(buf_size + 8);
    struct stat64 info;

    sprintf(path, "%s%s",
            p[a[x].path_no].path, a[x].fname);
    if( lstat64( path, &info ) == -1 )
    {
        print(MSG_371, path);
        exit(EXIT_FAILURE);
    }

    if (info.st_size != a[x].flen)
    {
        a[x].flag = ERR;
        a[x].flen = info.st_size;
    }

    f1 = open(path, O_RDONLY, 0);

    if ( f1 == -1)
    {
        fprintf (stderr,MSG_372, program_name, strerror (errno), path);
    }

    path[0] = '\0';
    sprintf(path, "%s%s",
            p[a[y].path_no].path, a[y].fname);

    if( lstat64( path, &info ) == -1 )
    {
        print(MSG_371, path);
        exit(EXIT_FAILURE);
    }

    if (info.st_size != a[y].flen)
    {
        a[y].flag = ERR;
        a[y].flen = info.st_size;
    }

    f2 = open(path, O_RDONLY, 0);

    if ( f2 == -1)
    {
        fprintf (stderr,MSG_373, program_name, strerror (errno), path);
    }

    if (a[x].flen == a[y].flen)
        flen = a[x].flen;
    else
        return CRC;

    z = ulldiv(flen, BUF_SIZE);
    br = z.quot;
    bs = BUF_SIZE;
    xb = ldiv(BUF_SIZE, ULLI_SIZE);
    bm = xb.quot;

    for (aa = 0; aa < br; aa++)
    {
        ret = read_file(f1, intbuf1, buf_size);
        ret = read_file(f2, intbuf2, buf_size);

        for (m = 0; m < bm; m++)
        {
            if(intbuf1[m] != intbuf2[m])
            {
                ctn = ctn + sizeof(long long int);
            }
        }
    }

    free(intbuf1);
    intbuf1 = NULL;
    free(intbuf2);
    intbuf2 = NULL;
    /* finished with flen * BUF_SIZE */

    if (ret == 0)
    {
        bs = (long int) z.rem;
        charbuf1 = (unsigned char *) xmalloc(bs + 8);
        charbuf2 = (unsigned char *) xmalloc(bs + 8);
        memset(charbuf1, 0x00, bs + 8);
        memset(charbuf2, 0x00, bs + 8);
        ret = read_file(f1, charbuf1, bs);
        ret = read_file(f2, charbuf2, bs);

        for (m = 0; m < bs; m++)
        {
            if(charbuf1[m] != charbuf2[m])
            {
                ctn = ctn + sizeof(char);
            }
        }

        free(charbuf1);
        charbuf1 = NULL;
        free(charbuf2);
        charbuf2 = NULL;
    }

    close(f1);
    close(f2);
    flen = flen * pct;
    ctn = ctn * 100;

    if (flen < ctn)
        return DIF;
//        return a[y].flag;
    else
        return DBL;
}
