/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * get_cmd_line.c
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MXcleaner.h"
#include "disp.h"
#include <unistd.h>
#include <getopt.h>

extern int exit_status;
extern int cfg[];
extern pathstruct *p_cfg;
extern char* path;
extern int flag[7];
extern int pct;
extern char *cfgfname;
extern char *cfgpath;
extern int p_cfg_no;

struct option long_options[] =
{
    {"quiet",       no_argument,        0, 'q'},
    {"clean",       no_argument,        0, 'c'},
    {"delete",      no_argument,        0, 'd'},
    {"percent",     required_argument,  0, 'p'},
    {"remove",      no_argument,        0, 'r'},
    {"test",        no_argument,        0, 't'},
    {"verbose",     no_argument,        0, 'v'},
    {"help",        no_argument,        0, 'h'},
    {"ignore",      no_argument,        0, 'i'},
    {"kill",        no_argument,        0, 'k'},
    {"update",      no_argument,        0, 'u'},
    {"save",        no_argument,        0, 's'},
    {0, 0, 0, 0}
};

void usage(int status);
int strip_files(char *);
int get_cmd_line(int, char **);


int get_cmd_line(int arg_c, char **arg_v)
{
    int c = 0;
    int i = 0;
    int x = 0;
    int acc = 0;
    int slen = 0;
    char *tail = NULL;

    while (1)
    {
        int optind = 0; /* getopt_long stores the option index here. */
        c = getopt_long (arg_c, arg_v, "qcdp:rtvhikus",
                         long_options, &optind);

        /* Detect the end of the options. */
        if (c == -1)
            break;

        switch (c)
        {
        case 0:
            /* If this option set a flag, do nothing else now. */
            if (long_options[optind].flag != 0)
                break;
            break;

        case 'c':
            cfg[CLEAN] = TRUE;
            break;

        case 'd':
            cfg[DEL] = TRUE;
            break;

        case 'h':
            cfg[HELP] = TRUE;
            break;

        case 'i':
            cfg[IGNORE] = TRUE;
            x++;
            break;

        case 'k':
            cfg[KILL] = TRUE;
            x++;
            break;

        case 'p':
            errno = 0;
            /* Parse it.  */
            pct = (int) strtol (optarg, &tail, 0);
            /* Add it in, if not overflow.  */
            if (errno)
                print (MSG_0101);
            if ((pct > 100) || (pct < 0))
            {
                print(MSG_0102, optarg);
                cfg[PCT] = FALSE;
            }
            else
            {
                cfg[PCT] = pct;
            }
            break;

        case 'q':
            cfg[QUIET] = TRUE;
            break;

        case 'r':
            cfg[REMOVE] = TRUE;
            break;

        case 'u':
            cfg[UPDATE] = TRUE;
            x++;
            break;

        case 's':
            cfg[SAVE] = TRUE;
            x++;
            break;

        case 't':
            cfg[TEST] = TRUE;
            break;

        case 'v':
            cfg[VERB] = TRUE;
            break;

        case '?':
            /* getopt_long already printed an error message. */
            break;

        default:
            usage (EXIT_FAILURE);
        }
    }

    if (cfg[HELP] == TRUE)
        usage(exit_status);

    if (x > 1)
    {
        print(MSG_0103);
        exit (0);
    }

    if (optind == arg_c)
    {
        p_cfg_no = strip_files(arg_v[i]);
        print(MSG_104, i, p_cfg[i].path);
    }
    else
    {
        while (optind < arg_c)
        {
            strcpy_s(path, _MAX_PATH, arg_v[optind]);
            acc = 0;
            acc = access(path, W_OK);
            if (acc == 0)
            {
                slen = strlen(path);
                if (path[slen - 1] != SLASH)
                {
                    path[slen] = SLASH;
                    path[slen + 1] = '\0';
                }
                strcpy_s(p_cfg[p_cfg_no].path, _MAX_PATH, path);
                p_cfg[p_cfg_no].path_no = p_cfg_no;
                p_cfg[p_cfg_no].plen = strlen(p_cfg[p_cfg_no].path);
//                print("Search Path [%d] = %s\n",
//                      p_cfg_no, p_cfg[p_cfg_no].path);
                p_cfg_no++;
                p_cfg = (pathstruct *) xrealloc(p_cfg, sizeof(pathstruct) * (p_cfg_no + 1));
            }

            if (i < p_cfg_no)
            {
                print(MSG_104, i, p_cfg[i].path); //40x
                i++;
            }

            optind++;
        }
    }

    print("\n");

    return (p_cfg_no);
}

