/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * main.c (MXcleaner)
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "MXcleaner.h"
#include "disp.h"
#include <unistd.h>

struct elapsed_t *elapsed_time(int);

/* extern variables */
extern int rec_no;
extern dirstruct *a;
extern dirstruct *b;
extern pathstruct *p;
extern pathstruct *p_cfg;
extern char *path;
extern int cfg[];
extern char* cfgpath;
extern int path_no;
extern int opt_no;
extern int mx_flag;
extern int nil_flag;

/* Function prototypes: */
int get_cmd_line(int, char **);
void *sort(dirstruct *, int );
void *sort_path(dirstruct *, int );
void mark_files(void);
void rename_files(void);
int init(void);
int show_list(int);
int yes_no(void);
int delete_files(int);
int remove_prefix(int);
int get_mx_data(int);
int get_mx_flen(int);
int creat_cfg(void);
int read_cfg(void);
int get_path_no(int);
int get_rec_no(int);
int put_path(int);
int put_rec(void);
int remove_utf8(int);
int read_dirs(void);
int get_nil_flen(int);
void show_options(void);
int strip_files(char *);
int create_cfg(void);

void make_hash(int);


int main(int argc, char **argv)
{
    int ret = 0;
    int i = 0;
    int acc = 0;
    int p_cfg_no = 0;
    char *prog_dir = NULL;
    char *ret_ptr = NULL;

    for (i = 0; i < argc; i++)
        print("argv[%d]: %s\n", i, argv[i]);
    print("\n");

    prog_dir = getcwd(ret_ptr, 0);
    printf("Program directory: %s\n\n", prog_dir);

    ret = init();
    if (ret != 0)
    {
        print(MSG_001);
        exit(EXIT_FAILURE);
    }

    if (ret == 0)
    {
        acc = access(cfgpath, R_OK | W_OK | F_OK);

        if ((argc == 1) && (acc != 0))
        {
            print(MSG_002, argc, argv[0]);
            strip_files(argv[0]);
        }
        else if (argc > 1)
        {
            p_cfg_no = get_cmd_line(argc, argv);
            if (p_cfg_no == 0)
                print(MSG_003);

            if (cfg[KILL] == TRUE)
            {
                ret = remove(cfgpath);
                if (ret != 0)
                    print(MSG_004, cfgpath);
                exit (ret);
            }
            else if (cfg[UPDATE] == TRUE)
            {
                ret = create_cfg();
                if (ret != 0)
                    print(MSG_005, cfgpath);
            }
            else if (cfg[SAVE] == TRUE)
            {
                ret = create_cfg();
                if (ret != 0)
                    print(MSG_006, cfgpath);
                exit (ret);
            }
            else if (cfg[IGNORE] == TRUE)
            {
                // ret = create_cfg(argc);
            }
        }
        else if (argc == 1)
        {
            p_cfg_no = read_cfg();
            if (p_cfg_no <= 0)
                print(MSG_007);
        }
    }
    free(cfgpath);

    print(MSG_009, PROGRAM_NAME, VERSION, AUTHORS, PROGRAM_NAME );
    sleep(2);
    show_options();

    for(i = 0; i < p_cfg_no; i++)
    {
        print(MSG_008, i, p_cfg[i].path);
    }

// elapsed_time(START);

    /* Read directories and populate file database */
    ret = get_path_no(p_cfg_no);
    ret= get_rec_no(p_cfg_no);
    ret = put_path(p_cfg_no);
    ret = put_rec();

    if (ret != 0)
        print(MSG_010);

    ret = remove_utf8(rec_no);
    if (ret != rec_no)
        printf("path_no: %3d rec_no: %6d\n", path_no, rec_no);

    if (cfg[VERB] == TRUE)
    {
        i = 0;
        for(i = 0; i < path_no; i++)
        {
            print("%6d %d  %s\n",
                  p[i].path_no,
                  p[i].plen,
                  p[i].path);
        }

        print(MSG_145);
        print(MSG_146);

        for(i = 0; i < rec_no; i++)
        {
            printf("%6d %6d %12lld   %s  %s%s\n",
                   a[i].path_no,
                   a[i].rec_no,
                   a[i].flen,
                   fl[a[i].flag],
                   p[a[i].path_no].path,
                   a[i].fname);
        }
    }

    /* Remove all __DUPLICATE__ and __DIF_DUPLICATE__
       prefixed from file names */
    ret = remove_prefix(rec_no);

    if (ret > 0)
        print(MSG_011);

    if (cfg[CLEAN] == FALSE)
    {
        /* Parse and flag __INCOMPLETE__ filelength */
        if (mx_flag == TRUE)
            ret = get_mx_flen(rec_no);

        /* Flag files with ZERO (NIL) flielength */
        if (nil_flag == TRUE)
            ret = get_nil_flen(rec_no);

        print(MSG_012,
              (unsigned long int) sizeof(dirstruct) * (rec_no * 2));

        /* Sort files */

        for (i = 0; i < rec_no; i++)
        {
            a[i].flen = (a[i].flen << 10);
            a[i].flen = (a[i].flen + a[i].path_no);
        }
        a = (dirstruct *) sort(a, rec_no);
        for (i = 0; i < rec_no; i++)
        {
            a[i].flen = (a[i].flen - a[i].path_no);
            a[i].flen = (a[i].flen >> 10);
        }

        if (cfg[VERB] == TRUE)
        {
            print(MSG_145);
            print(MSG_146);
            for(i = 0; i < rec_no; i++)
            {
                print("%d %6d %12lld %2u  %s  %s%s\n",
                      i,
                      a[i].rec_no,
                      a[i].flen,
                      a[i].path_no,
                      fl[a[i].flag],
                      p[a[i].path_no].path,
                      a[i].fname);
            }
        }

        /* Parse __INCOMPLETE__ file names */

        if (mx_flag == TRUE)
            ret = get_mx_data(rec_no);

        /* Mark files __DUPLICATE__, __DIFF_DUPLICATE__ or __ZERO_LENGTH__*/
        mark_files();

 elapsed_time(START);
        for(i = 0; i < rec_no; i++)
        {
            if (a[i].flag == CRC)
            {
                make_hash(i);
            }
        }

//        print_marked_file();
elapsed_time(STOP);

        if (cfg[VERB] == TRUE)
        {
            print(MSG_145);
            print(MSG_146);
            for(i = 0; i < rec_no; i++)
            {
                print("%6d %12lld %d  %s  %s%s\n",
                      a[i].rec_no,
                      a[i].flen,
                      a[i].path_no,
                      fl[a[i].flag],
                      p[a[i].path_no].path,
                      a[i].fname);
            }
        }

        rename_files();
        if (cfg[VERB] == TRUE)
        {
            print(MSG_145);
            print(MSG_146);
            for(i = 0; i < rec_no; i++)
            {
                print("%6d %12lld   %s  %s%s\n",
                      a[i].rec_no,
                      a[i].flen,
                      fl[a[i].flag],
                      p[a[i].path_no].path,
                      a[i].fname);
            }
            printf("\n\n");
            for(i = 0; i < rec_no; i++)
            {
                if (a[i].flag == CRC)
                {
                    print("%6d %12lld   %s  %s%s\n",
                          a[i].rec_no,
                          a[i].flen,
                          fl[a[i].flag],
                          p[a[i].path_no].path,
                          a[i].fname);
                }
            }
        }

        if (mx_flag == TRUE)
        {
            ret = show_list(MXD);
            if (ret > 0)
                delete_files(MXD);
        }
        ret = show_list(DBL);
        if (ret > 0)
            delete_files(DBL);

        if (nil_flag == TRUE)
        {
            ret = show_list(NDB);
            if (ret > 0)
                delete_files(NDB);
        }
        ret = show_list(DIF);

        if (cfg[REMOVE] == TRUE)
        {
            ret = put_rec();
            if (ret != 0)
                print(MSG_010);

            /* Remove all __DUPLICATE__ and __DIF_DUPLICATE__
               prefixed from file names */
            ret = remove_prefix(rec_no);
            if (ret > 0)
                print(MSG_011);
        }
    }

    ret = chdir(prog_dir);

    free(a);
    free(b);
    free(p);
    free(path);
    free (tmp);

    elapsed_time(STOP);

    exit(ret);
}
