/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * make_mx_hash.c (MXcleaner)
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MXcleaner.h"
#include "disp.h"

// #define BUF_SIZE 0x20000

extern char *path;
extern dirstruct *a;
extern pathstruct *p;
extern int mx_rec_no;

void mk_hash(unsigned char *pBuf, const unsigned int, unsigned char *);
void make_mx_hash(int);


void make_mx_hash(int i)
{
    FILE *fh = NULL;
    unsigned int len = 0;
    int ret = 0;
    long long int flen = 0;
    unsigned int blk_no = 0;
    unsigned char buf[BUF_SIZE];
    unsigned int hash[4];

    path[0] = '0';
    sprintf(path, "%s%s", p[a[i].path_no].path, a[i].fname);

    if((fh = fopen(path, "rb")) == NULL)
    {
        print(MSG_361);
        print(MSG_362);
        print("%6d %6d %12lld   %s %s%s\n",
               a[i].rec_no,
               a[i].mx_rec_no,
               a[i].flen,
               fl[a[i].flag],
               p[a[i].path_no].path,
               a[i].fname);
        print("\n;");
        return;
    }

    ret = fseek(fh, 0 , SEEK_END);
    flen = ftell(fh);
    ret = fseek(fh, 0, SEEK_SET);
    if (ret != 0)
        print(MSG_363);

    blk_no = flen / BUF_SIZE / 10;
    if(blk_no < 5)
        blk_no = 5;

    hash[0] = hash[1] = hash[2] = hash[3] = 0;
    len = fread(buf, 1, BUF_SIZE, fh);

    while(1)
    {
        mk_hash(buf, len, (unsigned char *) hash);
        if(len < BUF_SIZE)
            break;
        fseek(fh, BUF_SIZE * (blk_no - 1), SEEK_CUR);
        len = fread(buf, 1, BUF_SIZE, fh);
    }

    fclose(fh);
    a[i].mx_hash[0] = hash[0];
    a[i].mx_hash[1] = hash[1];
    a[i].mx_hash[2] = hash[2];
    a[i].mx_hash[3] = hash[3];
    mx_rec_no++;
    a[i].mx_rec_no = mx_rec_no;

}

void mk_hash(unsigned char *buf, const unsigned int len,
                    unsigned char *hash)
{
    unsigned char b, n;
    unsigned char *bp;
    int i;

    if(len <= 16)
        return;

    b = 0xC9;
    n = 0;
    bp = buf;

    while(bp < (buf + len - 16))
    {
        for (i = 0; i < 16; i++)
        {
            hash[i] += bp[i];
        }

        hash[0] ^= b;

        for(i = 1; i < 16; i++)
        {
            hash[i] ^= hash[i - 1];
        }

        b = n + hash[15];
        b = (b << 1) + ( b >> 7);
        bp += 0x10;
        n++;
    }
}
