/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * read_cfg.c (MXcleaner)
 * Copyright (C) Elmar Haag 2018 <linux@c5ace.com>
 *
 * MXcleaner is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MXcleaner is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MXcleaner.h"
#include "disp.h"
#include <getopt.h>
#include <stdio.h>
#include <unistd.h>

extern char *program_name;
extern int cfg[];
extern char *env;
extern char *cfgpath;
extern char *cfgfname;
extern pathstruct *p_cfg;
extern char *path;
extern int path_no;
extern int pct;
extern struct option long_options[];

int read_cfg(void);
int strrstr(char *, char *);
int strip_files(char *);
void usage(int status);
// int getline (char **, size_t *, FILE *);


int read_cfg(void)
{
    FILE *fh = NULL;
    char *lb = NULL;
    char *lc = NULL;
    char *tail = NULL;
    char pnum[4];
    size_t bs = 0;
    int p_cfg_no = 0;
    int a = 0;
    int b = 0;
    int c = 0;
    int i = 0;
    int ret = 0;
    int slen = 0;
    int optidx = 0;
    int line = 0;
    int cfgflag = 0;

    fh = fopen(cfgpath, "rt");
    if (fh == NULL)
    {
        print(MSG_121, cfgfname);
        return -1;
    }

    /* Fill Options */
    while ((ret = getline(&lb, &bs, fh)) != EOF)
    {
        if (lb[0] != '#')
        {
            slen = strlen(lb);
            if (lb[slen -1] == '\n')
            {
                lb[slen -1] = '\0';
            }
//            printf("long_options[%d].name %s\n", optidx,long_options[optidx].name);
            if (long_options[optidx].name != NULL)
            {
                lc = strstr(lb, long_options[optidx].name);

                if (lc != NULL)
                {
                    c = long_options[optidx].val;
                    cfgflag = strrstr(lc, "true");

                    switch (c)
                    {
                    case 0:

                        if (long_options[optidx].flag != 0)
                            break;
                        break;

                    case 'c':
                        cfg[CLEAN] = cfgflag;
                        break;

                    case 'd':
                        cfg[DEL] = cfgflag;
                        break;

                    case 'h':
                        cfg[HELP] = cfgflag;
                        break;

                    case 'p':
                        errno = 0;

                        while (lc[a] != '\0')
                        {
                            if (isdigit(lc[a]))
                            {
                                pnum[b] = lc[a];
                                b++;
                            }
                            a++;
                        }
                        pnum[b] = 0x0;

                        pct = (int) strtol (pnum, &tail, 0);

                        if (errno)
                            print (MSG_0101);
                        if ((pct > 100) || (pct < 0))
                        {
                            print(MSG_0102, optarg);
                            cfg[PCT] = FALSE;
                        }
                        else
                        {
                            cfg[PCT] = pct;
                        }
                        break;

                    case 'q':
                        cfg[QUIET] = cfgflag;
                        break;

                    case 'r':
                        cfg[REMOVE] = cfgflag;
                        break;

                    case 't':
                        cfg[TEST] = cfgflag;
                        break;

                    case 'v':
                        cfg[VERB] = cfgflag;
                        break;

                    case '?':

                        break;

                    default:
                        usage (EXIT_FAILURE);
                    }
                    print(MSG_123, line, c, lc);
                    optidx++;
                }

                if (lc == NULL)
                {
                    i = 0;
                    while (lb[i] !='\0')
                    {
                        if (lb[i] == SLASH)
                        {
                            ret = access(&lb[i], W_OK);
                            if (ret == 0)
                            {
                                strcpy_s(p_cfg[p_cfg_no].path, _MAX_PATH, &lb[i]);
                                p_cfg[p_cfg_no].path_no = p_cfg_no;
                                p_cfg[p_cfg_no].plen = strlen(p_cfg[p_cfg_no].path);
                                print("Search Path [%d] = %s\n",
                                      p_cfg_no, p_cfg[p_cfg_no].path);
                                p_cfg_no++;
                                p_cfg = (pathstruct *) xrealloc(p_cfg, sizeof(pathstruct) * (p_cfg_no + 1));

                                break;
                            }
                        }
                        i++;
                    }
                }
            }
        }
        line++;
    }

    /* Close MxCleaner.cfg */
    ret = fclose(fh);
    if (ret != 0)
    {
        print(MSG_124, cfgfname);
        exit (EXIT_FAILURE);
    }
    free (lb);
    return p_cfg_no;
}

/*
Commandline:
--test /home/elmar/yahoo2/Favorits-Test/ /home/elmar/yahoo2/Completed-Test/ /home/elmar/yahoo2/Download-Test/
*/
